package net.sf.sido.spring;

import net.sf.sido.pojo.DOFactory;
import net.sf.sido.support.DataFactories;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.parsing.BeanComponentDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.w3c.dom.Element;

public class SiDODOFactoryBeanDefinitionParser implements BeanDefinitionParser {

    public static final String ATTR_REF = "ref";
    public static final String ATTR_ID = "id";

    public static final String DEFAULT_DO_FACTORY_ID = "sidoDOFactory";
    public static final String DEFAULT_DATA_FACTORY_ID = "sidoDataFactory";

    @Override
    public BeanDefinition parse(Element element, ParserContext parserContext) {
        // ID
        String id = element.getAttribute(ATTR_ID);
        if (StringUtils.isBlank(id)) {
            id = DEFAULT_DO_FACTORY_ID;
        }

        // DOFactory definition
        BeanDefinitionBuilder doFactoryDefBuilder = BeanDefinitionBuilder.rootBeanDefinition(DOFactory.class);
        // DataFactory
        String ref = element.getAttribute(ATTR_REF);
        if (StringUtils.isNotBlank(ref)) {
            // Reference
            doFactoryDefBuilder.addConstructorArgReference(ref);
        } else {
            // New data factory
            BeanDefinitionBuilder dataFactoryDefBuilder = BeanDefinitionBuilder.rootBeanDefinition(DataFactories.class);
            dataFactoryDefBuilder.setFactoryMethod("typedFactory");
            parserContext.registerBeanComponent(new BeanComponentDefinition(dataFactoryDefBuilder.getRawBeanDefinition(), DEFAULT_DATA_FACTORY_ID));
            // Reference
            doFactoryDefBuilder.addConstructorArgReference(DEFAULT_DATA_FACTORY_ID);
        }
        // Scanning of schemas
        doFactoryDefBuilder.addConstructorArgValue(true);
        // Register the DOFactory
        parserContext.registerBeanComponent(new BeanComponentDefinition(doFactoryDefBuilder.getRawBeanDefinition(), id));

        // DOFactoryLoader definition
        BeanDefinitionBuilder doFactoryLoaderDefBuilder = BeanDefinitionBuilder.rootBeanDefinition(DOFactoryLoader.class);
        doFactoryLoaderDefBuilder.addPropertyReference("defaultDOFactory", id);
        doFactoryLoaderDefBuilder.setInitMethodName("load");
        parserContext.registerBeanComponent(new BeanComponentDefinition(doFactoryLoaderDefBuilder.getRawBeanDefinition(), "sidoDOFactoryLoader"));

        // OK
        return null;
    }
}
