package net.sf.sido.spring;

import net.sf.sido.support.DataFactories;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.parsing.BeanComponentDefinition;
import org.springframework.beans.factory.support.AbstractBeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.RootBeanDefinition;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

import java.util.ArrayList;
import java.util.List;

public class SiDOFactoryBeanDefinitionParser implements BeanDefinitionParser {

    public static final String ATTR_REF = "ref";
    public static final String ATTR_ID = "id";
    public static final String ELEM_PATH = "path";

    public static final String DEFAULT_DATA_FACTORY_ID = "sidoDataFactory";

    @Override
    public BeanDefinition parse(Element element, ParserContext parserContext) {
        // ID
        String id = element.getAttribute(ATTR_ID);
        if (StringUtils.isBlank(id)) {
            id = DEFAULT_DATA_FACTORY_ID;
        }
        // Factory reference ?
        String ref = element.getAttribute(ATTR_REF);
        if (StringUtils.isBlank(ref)) {
            // Creates the factory definition
            RootBeanDefinition factoryBeanDefinition = new RootBeanDefinition(DataFactories.class);
            factoryBeanDefinition.setFactoryMethodName("typedFactory");
            parserContext.registerBeanComponent(new BeanComponentDefinition(factoryBeanDefinition, id));
            ref = id;
        }
        // Loading paths
        List<String> paths = new ArrayList<String>();
        List<Element> ePaths = DomUtils.getChildElementsByTagName(element, ELEM_PATH);
        for (Element ePath : ePaths) {
            String path = DomUtils.getTextValue(ePath);
            if (StringUtils.isNotBlank(path)) {
                paths.add(path.trim());
            }
        }
        // Factory loader
        AbstractBeanDefinition factoryLoaderDefinition = BeanDefinitionBuilder.rootBeanDefinition(DataFactoryLoader.class)
                .addPropertyValue("paths", StringUtils.join(paths, " "))
                .addPropertyReference("dataFactory", ref)
                .setInitMethodName("load")
                .getRawBeanDefinition();
        parserContext.registerBeanComponent(new BeanComponentDefinition(factoryLoaderDefinition, "sidoDataFactoryLoader"));

        // OK
        return null;
    }
}
